-- ** NayMods.debug ** provides helpful debugging utilities.
-- Debugging can be enabled by enabling the DEBUG variable below.
-- ATTENTION:
--    This will cause some (very) spammy output messages in the chatframe.
-------------------------------------------------------------------------------

local NayMods = NayMods
local Utils = NayMods.utils
local DEBUG = false
local WARNINGS = true

local debugColor = "FF8914"
local warningColor = "FF3333"
local nayModsColor = "9999FF"

local Debug = {}

-----------------------------VARDUMP-------------------------------------------

--[[
Simple to string of an object.
]]
local function sstring(o)
    return '"' .. tostring(o) .. '"'
end

--[[
Recursively make a string representation of an object.
]]
local function recurse(o, indent)
    if indent == nil then indent = '' end
    local indent2 = indent .. '  '
    if type(o) == 'table' then
        local s = indent .. '{' .. '\n'
        local first = true
        for k,v in pairs(o) do
            if first == false then s = s .. ', \n' end
            if type(k) ~= 'number' then k = sstring(k) end
            s = s .. indent2 .. '[' .. k .. '] = ' .. recurse(v, indent2)
            first = false
        end
        return s .. '\n' .. indent .. '}'
    else
        return sstring(o)
    end
end

--[[
Dump any variable to the debug output.
Best suited to dump large tables to the output.
]]
function Debug:Dump(...)
    local args = {...}
    if #args > 1 then
        Dump(args)
    else
        self:WriteLine(recurse(args[1]))
    end
end

--[[
Writes a line to the debug output (default chat frame).
]]
function Debug:WriteLine(message, ...)
  if DEBUG and message then
    print(Utils:Color("NayMods: ", nayModsColor) .. Utils:Color("DEBUG " .. string.format(message, ...), debugColor))
  end
end

--[[
Writes a warning line to the debug output.
]]
function Debug:Warning(message, ...)
  if WARNINGS and message then
    print(Utils:Color("NayMods: ", nayModsColor) .. Utils:Color("WARNING " .. string.format(message, ...), warningColor))
  end
end

-----------------------------EXPORT--------------------------------------------

NayMods.debug = Debug
