-- ** NayMods.options ** provides all the Ace3 registry options.
-------------------------------------------------------------------------------

local NayMods = NayMods
local Debug = NayMods.debug
local Storage = NayMods.storage
local Message = NayMods.plugins.visuals.message
local Transmission = NayMods.transmission
local Media = LibStub("LibSharedMedia-3.0")
local ACR = LibStub("AceConfigRegistry-3.0")
local ACD = LibStub("AceConfigDialog-3.0")
local ACGUI = LibStub("AceGUI-3.0")

local Options = {}

--[[
Gets all the aditional options for all the registered boss mods in NayMods.
]]
local function getBossModOptions()
  local mods = {}

  for _, bossMod in NayMods:GetBossMods() do
    Debug:WriteLine("Options for %s", bossMod.name)
    mods[bossMod.name] = bossMod:GetOptions()
  end

  return mods
end

--[[
Creates the Ace3 options table.
]]
local function getAceOptions()
  return {
    name = "NayMods",
    type = "group",
    args = {
      general = {
        name = "General",
        type = "group",
        order = 1,
        args = {
          h1 = {
            type = "header",
            name = "General",
            width = "full",
            order = 1
          },
          generalDesc = {
            type = "description",
            name = "General options are default options that'll be used everywhere.\nYou can override these options on a boss per boss basis under the Bosses options.",
            fontSize = "medium",
            order = 2,
          },
          messagesGroup = {
            name = "Messages",
            type = "group",
            guiInline = true,
            order = 3,
            args = {
              messagesDesc = {
                type = "description",
                name = "Messages are displayed on screen and give you visual clues on what is happening.",
                fontSize = "medium",
                order = 1
              },
              defaultFont = {
                type = "select",
                dialogControl = "LSM30_Font",
                name = "Font",
                desc = "Font",
                values = AceGUIWidgetLSMlists.font,
                get = function()
                  return Storage.db.char.visuals.message.font
                end,
                set = function(self, key)
                  Storage.db.char.visuals.message.font = key
                end,
                order = 2
              },
              defaultFontSize = {
                type = "range",
                name = "Size",
                min = 8,
                max = 40,
                softMin = 8,
                softMax = 40,
                step = 1,
                bigStep = 4,
                get = function()
                  return Storage.db.char.visuals.message.fontSize
                end,
                set = function(info, value)
                  Storage.db.char.visuals.message.fontSize = value
                end,
                order = 3
              }
            }
          },
          versionCheck = {
            name = "Perform version check",
            desc = "Perform a raid-wide version check.",
            type = "execute",
            func = function()
              NayMods:RequestVersionsOverview()
            end,
            order = 4
          },
        }
      },
      bossMods = {
        name = "Bosses",
        type = "group",
        order = 2,
        args = getBossModOptions()
      },
      other = {
        name = "Other",
        type = "group",
        order = 3,
        args = {
          h1 = {
            type = "header",
            name = "Other stuff here.",
            width = "full"
          }
        }
      }
    }
  }
end

--[[
Initliazes the options.
]]
function Options:Initialize()

  local aceOptions = getAceOptions()

  ACR:RegisterOptionsTable("NayMods", aceOptions)
  ACD:SetDefaultSize("NayMods", 800, 675)

  SLASH_NAYMODS_CMD1 = "/nm"
  SLASH_NAYMODS_CMD2 = "/naymods"
  _G.SlashCmdList["NAYMODS_CMD"] = function()
    ACD:Open("NayMods")
  end

  Debug:WriteLine("Options initialized")
end

--[[
Notify the options to refresh themselves with new values.
]]
function Options:Refresh()
  ACR:NotifyChange("naymods")
end

-----------------------------EXPORT--------------------------------------------

NayMods.options = Options
