-- ** NayMods.transmission ** provides a handy transmission wrapper to send
-- messages all over NayMods.
-------------------------------------------------------------------------------

local NayMods = NayMods
local Debug = NayMods.debug
local AC = LibStub("AceComm-3.0")
local AS = LibStub("AceSerializer-3.0")

local PREFIX = "NAYMODS420"
local Transmission = {}

local callbackMap = {}

--[[
Pack a given key and data object into a serialized message.
]]
local function pack(key, data)
  return AS:Serialize({ key = key, data = data })
end

--[[
Unpack a serialized message back into a package with a key and data object.
]]
local function unpack(message)
  local _, package =  AS:Deserialize(message)

  return package
end

--[[
Callback handler when a message is received.
]]
local function handleCallback(prefix, message, dist, sender)
  local package = unpack(message)

  for k, f in pairs(callbackMap) do
    if k == package.key then
      f(k, package.data, dist, sender)
    end
  end
end

--[[
Register a callback to a given key.
]]
function Transmission:Register(key, callback)
  callbackMap[key] = callback
  AC:RegisterComm(PREFIX, function(...) handleCallback(...) end)
end

--[[
Send a transmission message with the given key and data
using the specified distribution channel and optionally a target.
]]
function Transmission:Send(key, data, dist, target)
  local package = pack(key, data)
  AC:SendCommMessage(PREFIX, package, dist, target)
end

-----------------------------EXPORT--------------------------------------------

NayMods.transmission = Transmission
