local NayMods = NayMods
local Debug = NayMods.debug
local Timer = NayMods.timer
local Utils = NayMods.utils
local Transmission = NayMods.transmission
local Options = NayMods.options
local Storage = NayMods.storage

local mod = NayMods:NewBossMod("Gorefiend", 1783)
mod:RegisterEnableMob(90199)


local digestTimers = {}
local hasDigest = {}

local DIGEST_EXAMPLE_MSG = "Bob: 35" .. "\n" .. "Johny: 20" .. "\n" .. "Lary: 9"

--[[
OnInit hook.
]]
function mod:OnInit()
end

--[[
Setup the default config for the bossmod.
]]
function mod:SetupDefaultConfig()
  self.defaultConfig = {
    char = {
      visuals = {
        [181295] = { -- digest timers
          xOffset = 0,
          yOffset = 200,
          fontSize = 30
        },
      }
    }
  }
end

--[[
Returns the Ace3 options sub-table for boss mod.
]]
function mod:GetOptions()
  local args = {
    digestHeader = {
      type = "header",
      name = "Digest",
      order = 1
    },
    digestMessageGroup = {
      type = "group",
      guiInline = true,
      name = "Digest timers",
      order = 2,
      args = {
        digestMessageEnabler = self
          :Message(181295, DIGEST_EXAMPLE_MSG, nil, true)
          :EnableOption(1),
        digestMessageFontSelector = self
          :Message(181295, DIGEST_EXAMPLE_MSG, nil, true)
          :FontSelector(2),
        digestMessageFontSizer = self
          :Message(181295, DIGEST_EXAMPLE_MSG, nil, true)
          :FontSizer(3),
        digestMessageTextAligner = self
          :Message(181295, DIGEST_EXAMPLE_MSG, nil, true)
          :TextAligner(4),
        digestMessageToggleAnchor = self
          :Message(181295, DIGEST_EXAMPLE_MSG, nil, true)
          :ToggleAnchorOption(5),
      }
    },
  }

  return {
    name = self.name,
    type = "group",
    args = args
  }
end

--[[
OnStart Hook.
]]
function mod:OnStart()
  digestTimers = {}
  hasDigest = {}

  self:Message(181295, DIGEST_EXAMPLE_MSG)
  self:Message(181295):RegisterOnUpdate("GOREFIEND_UPDATE_DIGEST_TIMERS" ,function(...) self:updateMsg(...) end)

  self:Subscribe("SPELL_AURA_APPLIED", "OnDigestApplied")
  self:Subscribe("SPELL_AURA_REMOVED", "OnDigestRemoved")
end

--[[
OnEngage hook.
]]
function mod:OnEngage()
end

--[[
OnWipe hook.
]]
function mod:OnWipe()
end

--[[
OnStop hook.
]]
function mod:OnStop()
  self:Unsubscribe("SPELL_AURA_APPLIED")
  self:Unsubscribe("SPELL_AURA_REMOVED")

  self:Message(181295):UnregisterOnUpdate("GOREFIEND_UPDATE_DIGEST_TIMERS")
  self:Message(181295):SetText(DIGEST_EXAMPLE_MSG)
  self:Message(181295):Hide()

  digestTimers = {}
  hasDigest = {}
end


function mod:OnDigestApplied(args)
  local player = args.destName

  if args.spellId == 181295 then
    local digestExpiresAt = mod:getDigestExpiration(181295, player)
    table.insert(digestTimers, digestExpiresAt)
    table.insert(hasDigest, player)
  end
end

function mod:OnDigestRemoved(args)
  local player = args.destName

  if args.spellId == 181295 then
    for k, v in pairs(hasDigest) do
      if v == player then
        table.remove(digestTimers, k)
        table.remove(hasDigest, k)
      end
    end
  end
end

function mod:updateMsg(...)
  local message = ...
  local txt = ""

  for k,v in pairs(hasDigest) do -- for every item in this table

      local playerName = v -- the players name
      local timer = string.format("%.1f",  digestTimers[k] - GetTime()) -- store the timer for the first person

      -- decide what color the timer should be
      -- anything above 15 seconds is green (default)

      local timerLabel = timer

      if tonumber(timer) > 15 then
        timerLabel = Utils:Color(timer, "00F500")
      elseif tonumber(timer) <= 15 and tonumber(timer) >= 7 then
        timerLabel = Utils:Color(timer, "FF8800")
      else
        timerLabel = Utils:Color(timer, "FF0000")
      end

      -- make a playername label with the players class color
      local playerNameLabel = Utils:ColorByClass(playerName)
      -- store the information on the display string
      txt = txt .. string.format("%s: %s", playerNameLabel, timerLabel) .. "\124n"


      if tonumber(timer) <= .1 then -- check if the timer is expired
          table.remove(digestTimers, k) -- remove the items from the table.
          table.remove(hasDigest, k)
      end


  end
  message:SetText(txt)
end

function mod:getDigestExpiration(spellId, playerName)
  local digest = GetSpellInfo(181295) or "Digest"
  local digestExpiresAt = select(7, UnitDebuff(playerName, digest)) or GetTime() + 35

  return digestExpiresAt
end
