local NayMods = NayMods
local Debug = NayMods.debug
local Timer = NayMods.timer
local Utils = NayMods.utils
local Transmission = NayMods.transmission
local Options = NayMods.options
local Storage = NayMods.storage

local mod = NayMods:NewBossMod("HellfireCouncil", 1798)

mod:RegisterEnableMob(92142) -- Blademaster Jubei'thos
mod:RegisterEnableMob(92144) -- Dia Darkwhisper
mod:RegisterEnableMob(92146) -- Gurtogg Bloodboil

local bladeMasterBelowThirty = false
local bladeMasterIsUp = true
local encounterIsDone = false

--[[
OnInit hook.
]]
function mod:OnInit()
end

--[[
Setup the default config for the bossmod.
]]
function mod:SetupDefaultConfig()
  self.defaultConfig = {
    char = {
      visuals = {
        [186993] = { -- Wicked strike alert
          xOffset = 0,
          yOffset = 200,
          fontSize = 32
        }
      }
    }
  }
end

--[[
Returns the Ace3 options sub-table for Archimonde.
]]
function mod:GetOptions()
  local args = {
    wickedStrikeAlertGroup = {
      type = "group",
      guiInline = true,
      name = "Wicked Strike Alert",
      order = 1,
      args = {
        wickedStrikeEnable = self
          :Message(186993, "Wicked strike incoming!", nil, true)
          :EnableOption(1),
        wickedStrikeFontSelector = self
          :Message(186993, "Wicked strike incoming!", nil, true)
          :FontSelector(2),
        wickedStrikeFontSizer = self
          :Message(186993, "Wicked strike incoming!", nil, true)
          :FontSizer(3),
        wickedStrikeToggleAnchor = self
          :Message(186993, "Wicked strike incoming!", nil, true)
          :ToggleAnchorOption(4)
      }
    }
  }

  return {
    name = self.name,
    type = "group",
    args = args
  }
end

--[[
OnStart Hook.
]]
function mod:OnStart()
  self:Subscribe("SPELL_DAMAGE", "OnWickedStrike")
  self:Subscribe("SPELL_ABSORBED", "OnWickedStrike")
  self:Subscribe("SPELL_CAST_SUCCESS", "OnMirrorImages")
  self:WatchHealth(92142, "OnBladeMasterBelowThirty")
end

--[[
OnEngage hook.
]]
function mod:OnEngage()
  bladeMasterBelowThirty = false
  bladeMasterIsUp = true
  bladeMasterIsDead = false
  mirrorImages = false
end

--[[
OnWipe hook.
]]
function mod:OnWipe()
    bladeMasterIsUp = false
    encounterIsDone = true
    mirrorImages = false
    self:hideWickedStrikeAlerts()
end

--[[
OnStop hook.
]]
function mod:OnStop()
  self:Unsubscribe("SPELL_DAMAGE")
  self:Unsubscribe("SPELL_ABSORBED")
  self:Unsubscribe("SPELL_CAST_SUCCESS")
  self:UnwatchHealth(92142)
end

function mod:hideWickedStrikeAlerts()
  self:HideMessage(186993)
end

function mod:wickedStrikeIncoming()
  if bladeMasterIsUp and bladeMasterBelowThirty then
    self:Message(186993, "Wicked strike incoming!", 5)
  else
    self:hideWickedStrikeAlerts()
  end
end

function mod:wickedStrikeCountDown()
  self:Message(186993, "Wicked strike in {D}", 8.3)
  Timer.After(8.3, function(...) self:wickedStrikeIncoming() end)
end

function mod:checkForBlademaster()
  if encounterIsDone then return end

  self:hideWickedStrikeAlerts()

  if not bladeMasterIsUp then
    local bladeMasterHP = self:GetUnitCurrentHealthPercentage(92142)
    if bladeMasterHP and bladeMasterHP > 0 then
      bladeMasterIsUp = true
      if bladeMasterBelowThirty then
        self:wickedStrikeCountDown()
      end
    else
      Timer.After(1, function(...) self:checkForBlademaster() end) -- check again in 1 second...
    end
  end
end


function mod:OnWickedStrike(args)
  if args.spellId == 186993 then
    self:wickedStrikeCountDown()
  end
end

function mod:OnMirrorImages(args)
  if args.spellId == 183885 and not bladeMasterIsUp then -- prevent doing this 10x when mirror images are cast.
    bladeMasterIsUp = false
    self:hideWickedStrikeAlerts()
    Timer.After(3, function(...) -- Put a little delay so checking for blademaster only kicks in when he's really gone.
      self:checkForBlademaster()
    end)
  end
end

function mod:OnBladeMasterBelowThirty(currentHealthPercent)
  if currentHealthPercent and currentHealthPercent <= 30 and not bladeMasterBelowThirty then
    bladeMasterBelowThirty = true
    self:wickedStrikeCountDown()
  end
end
